<?php

namespace Drupal\instagram_sync\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\instagram_sync\ApiTypePluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Instagram Sync settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Name of the config object.
   *
   * @var string
   */
  const CONFIG_NAME = 'instagram_sync.settings';

  /**
   * API Type plugin manager.
   *
   * @var \Drupal\instagram_sync\ApiTypePluginManager
   */
  protected ApiTypePluginManager $apiTypePluginManager;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'instagram_sync_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.api_type')
    );
  }

  /**
   * Constructs a SettingsForm.
   *
   * @param \Drupal\instagram_sync\APITypePluginManager $api_type_plugin_manager
   *   Api Type Plugin manager.
   */
  public function __construct(ApiTypePluginManager $api_type_plugin_manager) {
    $this->apiTypePluginManager = $api_type_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::CONFIG_NAME);

    $options = [];
    foreach ($this->apiTypePluginManager->getDefinitions() as $id => $definition) {
      $options[$id] = $definition['label'];
    }

    $form['wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'settings-form-wrapper'],
    ];

    $form['api_type_plugin'] = [
      '#type' => 'select',
      '#options' => $options,
      '#title' => $this->t('Choose API Type plugin'),
      '#required' => TRUE,
      '#default_value' => $config->get('api_type_plugin') ?? NULL,
    ];

    $form['access_token'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Access Token'),
      '#description' => $this->t('Enter your long-term access token here.'),
      '#required' => TRUE,
      '#default_value' => $config->get('access_token') ?? '',
      '#ajax' => [
        'callback' => '::profileIdAjaxCallback',
        'wrapper' => 'settings-form-wrapper',
        'event' => 'change',
      ],
    ];

    $profileId = $config->get('profile_id') ?? '';
    $form['profile_id'] = [
      '#type' => 'hidden',
      '#default_value' => $profileId,
      '#attributes' => ['id' => 'profile-value'],
    ];

    $markup = '<div id="profile-id-wrapper">' . ($profileId ? $this->t("Your access token is correct. Profile ID is: @profileId", ['@profileId' => $profileId]) : '') . '</div>';
    $form['profile_id_markup'] = [
      '#type' => 'markup',
      '#markup' => $markup,
    ];

    $form['settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Global settings'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
    ];

    /*$form['settings']['video_ignore'] = [
    '#type' => 'checkbox',
    '#title' => $this->t('Ignore video download on import.'),
    '#default_value' => $config->get('video_ignore'),
    ];*/

    /*$form['settings']['carousel_ignore'] = [
    '#type' => 'checkbox',
    '#title' => $this->t('Ignore other pictures for carousels.
    Load just first image.'),
    '#default_value' => $config->get('carousel_ignore'),
    ];*/

    $form['settings']['limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Quantity limit of posts to import'),
      '#description' => $this->t('Please enter an integer value. Leave empty to import all posts.'),
      '#required' => TRUE,
      '#min' => 3,
      '#step' => 1,
      '#default_value' => $config->get('limit') ?? '',
    ];

    $form['token'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Refresh settings'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
    ];

    $form['token']['token_refresh_auto_update'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable access token auto-refresh'),
      '#default_value' => $config->get('token_refresh_auto_update'),
    ];

    $form['token']['token_refresh_period'] = [
      '#type' => 'number',
      '#title' => $this->t('Token refresh period'),
      '#description' => $this->t('Set the token refresh period in days. Maximum: 59 days.'),
      '#states' => [
        'visible' => [
          ':input[name="token_refresh_auto_update"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="token_refresh_auto_update"]' => ['checked' => TRUE],
        ],
      ],
      '#default_value' => $config->get('token_refresh_period'),
      '#min' => 1,
      '#max' => 59,
      '#step' => 1,
    ];

    $form['cron_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('CRON settings'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
    ];

    $form['cron_settings']['cron_auto_update'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable auto-sync'),
      '#default_value' => $config->get('cron_auto_update'),
    ];

    $form['cron_settings']['cron_refresh_period'] = [
      '#type' => 'number',
      '#title' => $this->t('Auto-sync period'),
      '#description' => $this->t('Set the auto-sync period in days. Maximum: 30 days.'),
      '#states' => [
        'visible' => [
          ':input[name="cron_auto_update"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="cron_auto_update"]' => ['checked' => TRUE],
        ],
      ],
      '#default_value' => $config->get('cron_refresh_period'),
      '#min' => 1,
      '#max' => 30,
      '#step' => 1,
    ];

    $form = parent::buildForm($form, $form_state);
    $form['actions']['instagram_import'] = [
      '#type' => 'submit',
      '#value' => $this->t('Import instagram posts'),
      '#submit' => ['::processBatch'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $value = $form_state->getValue('profile_id');
    if (!$value) {
      $form_state->setErrorByName('access_token', $this->t('Please enter valid access token.'));
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config(static::CONFIG_NAME)
      ->set('api_type_plugin', $form_state->getValue('api_type_plugin'))
      ->set('access_token', $form_state->getValue('access_token'))
      ->set('token_refresh_auto_update', $form_state->getValue('token_refresh_auto_update'))
      ->set('token_refresh_period', $form_state->getValue('token_refresh_period'))
      ->set('profile_id', $form_state->getValue('profile_id'))
      // ->set('video_ignore', $form_state->getValue('video_ignore'))
      // ->set('carousel_ignore', $form_state->getValue('carousel_ignore'))
      ->set('limit', $form_state->getValue('limit'))
      ->set('cron_auto_update', $form_state->getValue('cron_auto_update'))
      ->set('cron_refresh_period', $form_state->getValue('cron_refresh_period'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Starts batch import process on button click.
   */
  public function processBatch(array &$form, FormStateInterface &$form_state) {
    $plugin = $form_state->getValue('api_type_plugin');
    $postLimit = $form_state->getValue('limit');
    instagram_sync_request_batch($plugin, $postLimit);
  }

  /**
   * Checks profile id by access_token.
   *
   * @param array $form
   *   Form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Ajax response.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   *   Exception.
   */
  public function profileIdAjaxCallback(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();
    $accessToken = $form_state->getValue('access_token');
    $pluginId = $form_state->getValue('api_type_plugin');
    $output = $this->t("Your access token is wrong. Please, verify.");
    $profileId = '';

    /** @var \Drupal\instagram_sync\ApiTypePluginManager $pluginManager */
    $pluginManager = \Drupal::service('plugin.manager.api_type');

    if ($pluginManager->hasDefinition($pluginId)) {
      /** @var \Drupal\instagram_sync\APITypePluginBase $pluginInstance */
      $pluginInstance = $pluginManager->createInstance($pluginId);
      $profile = $pluginInstance->getProfileData(['access_token' => $accessToken]);
      $profileId = $profile['id'] ?? '';
      if ($profileId) {
        $output = $this->t("Your access token is correct. Profile ID is: @profileId", ['@profileId' => $profileId]);
      }
    }
    $response->addCommand(new InvokeCommand('#profile-value', 'val', [$profileId]));
    $response->addCommand(new InvokeCommand('#profile-id-wrapper', 'text', [$output]));

    return $response;
  }

}
