<?php

namespace Drupal\instagram_sync\Services;

use Drupal\instagram_sync\ApiTypePluginManager;
use Drupal\instagram_sync\Exception\InstagramSyncException;
use Drupal\instagram_sync\Utility\InstagramAPIMedia;
use Drupal\Core\State\StateInterface;

/**
 * Service description.
 */
class InstagramSyncService {

  /**
   * API type plugin manager service.
   *
   * @var \Drupal\instagram_sync\ApiTypePluginManager
   *   API type plugin manager service variable.
   */
  protected ApiTypePluginManager $apiTypePluginManager;

  /**
   * State service.
   *
   * @var \Drupal\Core\State\StateInterface
   *   State interface service variable.
   */
  protected StateInterface $state;

  /**
   * Query params needs to be saved for the next query.
   */
  protected array $nextQueryParams = [];

  /**
   * Constructs a new CustomService object.
   */
  public function __construct(ApiTypePluginManager $apiTypePluginManager, StateInterface $state) {
    $this->apiTypePluginManager = $apiTypePluginManager;
    $this->state = $state;
  }

  /**
   * Gets refresh token from the instagram API.
   */
  public function refreshAccess(string $pluginId, array $data): array|null {
    if (!$this->apiTypePluginManager->hasDefinition($pluginId)) {
      return NULL;
    }

    /** @var \Drupal\instagram_sync\APITypePluginBase $pluginInstance */
    $pluginInstance = $this->apiTypePluginManager->createInstance($pluginId);
    try {
      $result = $pluginInstance->refreshAccess($data);
      $this->state->set('instagram_sync_last_token_update_run', time());

      return $result;
    }
    catch (\Exception $e) {
      watchdog_exception('error', $e);
    }

    return NULL;
  }

  /**
   * Gets media object from the API.
   */
  public function getPosts(string $pluginId, $params = []): array {
    $result = [];
    if (!$this->apiTypePluginManager->hasDefinition($pluginId)) {
      return [];
    }

    /** @var \Drupal\instagram_sync\APITypePluginBase $pluginInstance */
    $pluginInstance = $this->apiTypePluginManager->createInstance($pluginId);
    try {
      $data = $pluginInstance->importMedia($params);
      if (isset($data->data)) {
        foreach ($data->data as $media) {
          try {
            $result[] = new InstagramAPIMedia($media);
          }
          catch (InstagramSyncException $exception) {
            watchdog_exception('error', $exception);
          }
        }
        $this->state->set('instagram_sync_last_import_run', time());
      }

      if (!empty($data->paging->cursors->after)) {
        $this->addNextQueryParam('after', $data->paging->cursors->after);
      }

      return $result;
    }
    catch (\Exception $e) {
      watchdog_exception('error', $e);
    }

    return [];
  }

  /**
   * Saves GET params fot the next request.
   */
  protected function addNextQueryParam(string $param, string $value): void {
    $this->nextQueryParams[$param] = $value;
  }

  /**
   * Gets GET params fot the current request.
   */
  public function getNextQueryParams(): array {
    return $this->nextQueryParams;
  }

}
